@@ -1,15 +1,24 @@
 import os
 import argparse
 import json
+import logging
+import shutil
+import tempfile
 import pytest
 import responses
 from munch import Munch
 
 import copr
-from copr.exceptions import CoprUnknownResponseException, CoprBuildException
-from copr_cli.main import no_config_warning, FrontendOutdatedCliException
+from copr.exceptions import (
+    CoprBuildException,
+    CoprConfigException,
+    CoprUnknownResponseException,
+)
+from copr.v3.exceptions import CoprAuthException
+from cli_tests_lib import config as mock_config, mock, MagicMock
+from copr_cli import main
+from copr_cli.main import FrontendOutdatedCliException
 
-from cli_tests_lib import mock, MagicMock, config as mock_config
 
 def exit_wrap(value):
     if type(value) == int:
@@ -29,14 +38,29 @@ def exit_wrap(value):
 # log = logging.getLogger()
 # log.info("Logger initiated")
 
-from copr_cli import main
+class TestCliWrapper:
+    tmpdir = None
+    configfile = None
 
-@mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
-def test_parse_name(config_from_file):
-    cmd = main.Commands(config_path=None)
-    assert cmd.parse_name("foo") == (None, "foo")
-    assert cmd.parse_name("frostyx/foo") == ("frostyx", "foo")
-    assert cmd.parse_name("@copr/foo") == ("@copr", "foo")
+    def setup_method(self, _method):
+        self.tmpdir = tempfile.mkdtemp(prefix="test-cli-name-parseing")
+        self.configfile = os.path.join(self.tmpdir, "config")
+
+        with open(self.configfile, 'w') as fd:
+            fd.write("[copr-cli]\n")
+            fd.write("copr_url = https://xyz/\n")
+            fd.write("token = xyz\n")
+            fd.write("username = jdoe\n")
+            fd.write("login = login\n")
+
+    def teardown_method(self, _method):
+        shutil.rmtree(self.tmpdir)
+
+    def test_parse_name(self):
+        cmd = main.Commands(config_path=self.configfile)
+        assert cmd.parse_name("foo") == ("jdoe", "foo")
+        assert cmd.parse_name("frostyx/foo") == ("frostyx", "foo")
+        assert cmd.parse_name("@copr/foo") == ("@copr", "foo")
 
 
 @mock.patch('copr.v3.proxies.build.BuildProxy.get')
@@ -145,21 +169,15 @@ def test_error_copr_unknown_response(config_from_file, action_status, capsys):
     assert error_msg in stderr
 
 
+@responses.activate
+@mock.patch('copr.v3.proxies.build.BuildProxy.cancel')
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
-def test_cancel_build_no_config(config_from_file, capsys):
-    config_from_file.side_effect = copr.v3.CoprNoConfigException("Dude, your config is missing")
-
-    with pytest.raises(SystemExit) as err:
-        main.main(argv=["cancel", "123400"])
-
-    assert exit_wrap(err.value) == 6
-    out, err = capsys.readouterr()
-
-    assert "Error: Operation requires api authentication" in err
-    assert "File '~/.config/copr' is missing or incorrect" in err
-
-    expected_warning = no_config_warning.format("~/.config/copr", "Dude, your config is missing")
-    assert expected_warning in err
+def test_cancel_build_no_config(_cff, build_proxy_cancel, capsys):
+    response_status = "foobar"
+    build_proxy_cancel.return_value = MagicMock(state=response_status)
+    main.main(argv=["cancel", "123"])
+    out, _ = capsys.readouterr()
+    assert "{0}\n".format(response_status) in out
 
 
 @mock.patch('copr.v3.proxies.build.BuildProxy.cancel')
@@ -179,38 +197,33 @@ def read_res(name):
     return open(filepath).read()
 
 
-@mock.patch('copr_cli.main.config_from_file')
+@responses.activate
+@mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
 @mock.patch('copr.v3.proxies.project.ProjectProxy.get_list')
-def test_list_project(get_list, config_from_file, capsys):
+def test_list_project(get_list, _config_from_file, capsys):
     response_data = json.loads(read_res('list_projects_response.json'))
     expected_output = read_res('list_projects_expected.txt')
 
-    # no config
-    config_from_file.side_effect = copr.v3.CoprNoConfigException("Dude, your config is missing")
     control_response = [Munch(x) for x in response_data]
     get_list.return_value = control_response
 
     main.main(argv=["list", "rhscl"])
-    out, err = capsys.readouterr()
+    out, _ = capsys.readouterr()
     assert expected_output in out
 
-    expected_warning = no_config_warning.format("~/.config/copr", "Dude, your config is missing")
-    assert expected_warning in err
-
 @responses.activate
 @mock.patch("copr_cli.main.next_page")
-@mock.patch('copr_cli.main.config_from_file')
-def test_list_builds(config_from_file, next_page, capsys):
+@mock.patch('configparser.ConfigParser.read')
+def test_list_builds(read, next_page, capsys):
+    read.return_value = []
     response_data = json.loads(read_res('list_builds_response.json'))
     expected_output = read_res('list_builds_expected.txt')
 
     responses.add(
         responses.GET,
-        'http://copr.fedoraproject.org/api_3/build/list',
+        'https://copr.fedorainfracloud.org/api_3/build/list',
         json=response_data, status=202)
 
-    # no config
-    config_from_file.side_effect = copr.v3.CoprNoConfigException("Dude, your config is missing")
     next_page.return_value = None
 
     main.main(argv=["list-builds", "praiskup/ping"])
@@ -219,18 +232,17 @@ def test_list_builds(config_from_file, next_page, capsys):
 
 @responses.activate
 @mock.patch("copr_cli.main.next_page")
-@mock.patch('copr_cli.main.config_from_file')
-def test_list_packages(config_from_file, next_page, capsys):
+@mock.patch('configparser.ConfigParser.read')
+def test_list_packages(read, next_page, capsys):
+    read.return_value = []
     response_data = json.loads(read_res('list_packages_response.json'))
     expected_output = read_res('list_packages_expected.json')
 
     responses.add(
         responses.GET,
-        'http://copr.fedoraproject.org/api_3/package/list',
+        'https://copr.fedorainfracloud.org/api_3/package/list',
         json=response_data, status=202)
 
-    # no config
-    config_from_file.side_effect = copr.v3.CoprNoConfigException("Dude, your config is missing")
     next_page.return_value = None
 
     main.main(argv=["list-packages", "praiskup/ping"])
@@ -238,27 +250,23 @@ def test_list_packages(config_from_file, next_page, capsys):
     assert json.loads(out) == json.loads(expected_output)
 
 @responses.activate
-@mock.patch('copr_cli.main.config_from_file')
-def test_get_package(config_from_file, capsys):
-
+@mock.patch('configparser.ConfigParser.read')
+def test_get_package(read, capsys):
+    read.return_value = []
     response_data = json.loads(read_res('get_package_response.json'))
     response_build_data = json.loads(read_res('get_package_response_builds.json'))
     expected_output = read_res('get_package_expected.json')
 
     responses.add(
         responses.GET,
-        'http://copr.fedoraproject.org/api_3/package',
+        'https://copr.fedorainfracloud.org/api_3/package',
         json=response_data, status=202)
 
     responses.add(
         responses.GET,
-        'http://copr.fedoraproject.org/api_3/build/list',
+        'https://copr.fedorainfracloud.org/api_3/build/list',
         json=response_build_data, status=202)
 
-
-    # no config
-    config_from_file.side_effect = copr.v3.CoprNoConfigException("Dude, your config is missing")
-
     main.main(argv=[
         "get-package", "--name", "binutils", "--with-all-builds",
         "praiskup/autoconf-2.71-attempts", "--with-latest-succeeded-build",
@@ -266,26 +274,40 @@ def test_get_package(config_from_file, capsys):
     out, _ = capsys.readouterr()
     assert json.loads(out) == json.loads(expected_output)
 
-@mock.patch('copr_cli.main.config_from_file')
-def test_list_project_no_username(config_from_file, capsys):
-    config_from_file.side_effect = copr.v3.CoprNoConfigException()
 
+@mock.patch('copr.v3.proxies.BaseProxy.auth_username')
+@mock.patch('copr_cli.main.config_from_file')
+def test_list_project_no_username(ac, cff, capsys):
+    """
+    Config unset (and gssapi ON by default in cli)
+    """
+    ac.side_effect = CoprAuthException("gssapi fail")
+    cff.side_effect = CoprConfigException("foo")
     with pytest.raises(SystemExit) as err:
         main.main(argv=["list"])
-
-    assert exit_wrap(err.value) == 6
-    out, err = capsys.readouterr()
-    assert "Pass username to command or create `~/.config/copr`" in err
+    assert exit_wrap(err.value) == 7
+    _, err = capsys.readouterr()
+    msg = "Operation requires API authentication. See the 'AUTHENTICATION'"
+    assert msg in err
 
 
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
 def test_list_project_no_username2(config_from_file, capsys):
+    config_from_file.return_value = {
+        "username": None,
+        "copr_url": "http://copr/",
+        "login": "",
+        "token": "test_token_XXX",
+        "gssapi": False,
+    }
     with pytest.raises(SystemExit) as err:
         main.main(argv=["list"])
 
     assert exit_wrap(err.value) == 6
     out, err = capsys.readouterr()
-    assert "Pass username to command or add it to `~/.config/copr`" in err
+    exp = "This operation tries to detect your username, but it is not " + \
+          "possible to find it in configuration, and GSSAPI is disabled"
+    assert exp in err
 
 
 @mock.patch('copr.v3.proxies.project.ProjectProxy.get_list')
@@ -316,10 +338,18 @@ def test_debug_by_status_response(config_from_file, build_proxy_get, capsys):
     response_status = "foobar"
     build_proxy_get.return_value = MagicMock(state=response_status)
 
+    # The main() --debug option affects the logger configuration, and capsys has
+    # trouble to restore the state back after the test case.  Let's help it.
+    # More info: # https://github.com/pytest-dev/pytest/issues/14
+    # This handler backup-restore appears to be needed on EL7 only.
+    log = logging.getLogger()
+    old_handlers = list(log.handlers)
+
     main.main(argv=["--debug", "status", "123"])
     stdout, stderr = capsys.readouterr()
     assert "{0}\n".format(response_status) in stdout
     assert "Debug log enabled " in stderr
+    log.handlers = old_handlers
 
 
 def test_status_response_no_args(capsys):
@@ -435,7 +465,7 @@ def test_create_project(config_from_file, project_proxy_add, capsys):
     args, kwargs = project_proxy_add.call_args
     assert kwargs == {
         "auto_prune": True,
-        "ownername": None, "persistent": False, "projectname": "foo", "description": "desc string",
+        "ownername": "jdoe", "persistent": False, "projectname": "foo", "description": "desc string",
         "instructions": "instruction string", "chroots": ["f20", "f21"],
         "additional_repos": ["repo1", "repo2"],
         "unlisted_on_hp": None, "devel_mode": None, "enable_net": False,
@@ -446,8 +476,10 @@ def test_create_project(config_from_file, project_proxy_add, capsys):
         "module_hotfixes": False,
         "fedora_review": False,
         "appstream": True,
+        "runtime_dependencies": None,
+        "packit_forge_projects_allowed": None,
     }
-    assert stdout == "New project was successfully created.\n"
+    assert stdout == "New project was successfully created: http://copr/coprs/jdoe/foo/\n"
 
 
 @mock.patch('copr.v3.proxies.project.ProjectProxy.add')
@@ -464,7 +496,7 @@ def test_create_project_with_isolation(_config_from_file, project_proxy_add, cap
     kwargs = project_proxy_add.call_args[1]
     assert stderr == ''
     assert kwargs["isolation"] == "simple"
-    assert stdout == "New project was successfully created.\n"
+    assert stdout == "New project was successfully created: http://copr/coprs/jdoe/foo/\n"
 
 
 @mock.patch('copr.v3.proxies.project_chroot.ProjectChrootProxy.edit')
@@ -526,7 +558,7 @@ def test_create_multilib_project(config_from_file, project_proxy_add, capsys):
     args, kwargs = project_proxy_add.call_args
     assert kwargs == {
         "auto_prune": True,
-        "ownername": None, "persistent": False, "projectname": "foo",
+        "ownername": "jdoe", "persistent": False, "projectname": "foo",
         "description": None,
         "instructions": "instruction string",
         "chroots": ["fedora-rawhide-x86_64", "fedora-rawhide-i386"],
@@ -539,11 +571,13 @@ def test_create_multilib_project(config_from_file, project_proxy_add, capsys):
         "module_hotfixes": False,
         "fedora_review": False,
         "appstream": True,
+        "runtime_dependencies": None,
+        "packit_forge_projects_allowed": None,
     }
-    assert stdout == "New project was successfully created.\n"
+    assert stdout == "New project was successfully created: http://copr/coprs/jdoe/foo/\n"
 
 
-@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
 @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
 @mock.patch('copr_cli.main.Commands._watch_builds')
@@ -561,7 +595,7 @@ def test_create_build_no_wait_ok(watch_builds, config_from_file, create_from_url
     assert not watch_builds.called
 
 
-@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
 @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
 @mock.patch('copr_cli.main.Commands._watch_builds')
@@ -581,7 +615,7 @@ def test_create_build_no_wait_error(watch_builds, config_from_file,create_from_u
 
 
 @mock.patch('copr_cli.main.time')
-@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
 @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
 @mock.patch('copr.v3.proxies.build.BuildProxy.get')
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
@@ -600,7 +634,7 @@ def test_create_build_wait_succeeded_no_sleep(config_from_file, build_proxy_get,
     assert not mock_time.sleep.called
 
 
-@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
 @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
 @mock.patch('copr.v3.proxies.build.BuildProxy.get')
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
@@ -619,7 +653,7 @@ def test_create_build_wait_error_status(config_from_file, build_proxy_get, creat
     assert "Watching build" in stdout
 
 
-@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
 @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
 @mock.patch('copr.v3.proxies.build.BuildProxy.get')
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
@@ -638,7 +672,7 @@ def test_create_build_wait_unknown_build_status(config_from_file, build_proxy_ge
     assert "Watching build" in stdout
 
 
-@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+@mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
 @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
 @mock.patch('copr.v3.proxies.build.BuildProxy.get')
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
@@ -660,7 +694,7 @@ def test_create_build_wait_keyboard_interrupt(config_from_file, build_proxy_get,
 @mock.patch('copr_cli.main.config_from_file', return_value=mock_config)
 class TestCreateBuild(object):
 
-    @mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+    @mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
     @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
     @mock.patch('copr.v3.proxies.build.BuildProxy.get')
     def test_create_build_wait_succeeded_complex(self, build_proxy_get, create_from_url, auth_check,
@@ -697,7 +731,7 @@ class TestCreateBuild(object):
         assert "Watching build" in stdout
         assert len(mock_time.sleep.call_args_list) == 3
 
-    @mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=True)
+    @mock.patch('copr.v3.proxies.BaseProxy.auth_check', return_value=Munch(name="test"))
     @mock.patch('copr.v3.proxies.build.BuildProxy.create_from_url')
     @mock.patch('copr.v3.proxies.build.BuildProxy.get')
     def test_create_build_wait_failed_complex(self, build_proxy_get, create_from_url, auth_check,
@@ -745,7 +779,6 @@ def test_edit_permissions_output(_config, action):
         args = action.call_args[0][0]
         assert args.permissions == expected_output
 
-
     with pytest.raises(SystemExit) as err:
         main.main(['edit-permissions'])
     assert err.value.code == 2
